@@ -142,20 +142,20 @@ class Point:
                 #ineqs.append('intConstraints')
             for field in ineqs:
                 fv = array(getattr(self, field)())#.flatten()
-                if fv.size > 0:
+                if fv.size > 0 and not all(isnan(fv)):
                     val_max = nanmax(fv, fv.ndim-1)
                     r = where(r < val_max, val_max, r) # handles nan ok
-                    if not self.isMultiArray and not isnan(val_max):
+                    if not self.isMultiArray:
                         ind_max = where(fv==val_max)[0][0]
                         if r < val_max:
                             r, ind, fname = val_max, ind_max, field
             for field in eqs:
                 fv = array(getattr(self, field)())#.flatten()
-                if fv.size > 0:
+                if fv.size > 0 and not all(isnan(fv)):
                     fv = abs(fv)
                     val_max = nanmax(fv, fv.ndim-1)
                     r = where(r < val_max, val_max, r) # handles nan ok
-                    if not self.isMultiArray and not isnan(val_max):
+                    if not self.isMultiArray:
                         ind_max = argmax(fv)
                         #val_max = fv[ind_max]
                         if r < val_max:
