@@ -105,16 +105,16 @@ class nonLinFuncs:
                                                funcs[i].D(x, fixedVars = p.fixedVars, useSparse=p.useSparse, fixedVarsScheduleID=p._FDVarsID, exactShape=True), 
                                                useSparse=p.useSparse, func=funcs[i], point=x)) \
                       for i in range(len(funcs))]
-            else:
+            else: # function, not derivative
+                _kw = {'fixedVarsScheduleID': p._FDVarsID}
                 if p.freeVars is None or (p.fixedVars is not None and len(p.freeVars) < len(p.fixedVars)):
-                    funcs2 = [(lambda x, i=i: \
-                               funcs[i]._getFuncCalcEngine(x, Vars = p.freeVars, fixedVarsScheduleID=p._FDVarsID))\
-                      for i in range(len(funcs))]
+                    _kw['Vars'] = p.freeVars
                 else:
-                    funcs2 = [(lambda x, i=i: \
-                               funcs[i]._getFuncCalcEngine(x, fixedVars = p.fixedVars, fixedVarsScheduleID=p._FDVarsID))\
+                    _kw['fixedVars'] = p.fixedVars
+                funcs2 = [(lambda x, i=i: \
+                               funcs[i]._getFuncCalcEngine(x, **_kw))\
                       for i in range(len(funcs))]
-        else:
+        else: # not FD model
             funcs2 = funcs
             
         if ind is None: 
@@ -158,8 +158,10 @@ class nonLinFuncs:
 #                        xx.append((oov, multiarray(x[:, counter: counter + s].flatten() if s == 1 else x[:, counter: counter + s])))
                         counter += s
                     X = oopoint(xx)
-                    X.update(p.dictOfFixedFuncs)
+                    X.dictOfFixedFuncs = p.dictOfFixedFuncs
+                    X._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                     X.maxDistributionSize = p.maxDistributionSize
+                    X._dictOfStochVars = p._dictOfStochVars
                     X._p = p
                     if x.ndim > 1:
                         X.isMultiPoint = True
@@ -168,9 +170,12 @@ class nonLinFuncs:
                     XX = [p._vector2point(x[i]) for i in range(nXvectors)]
                     for _X in XX: 
                         _X._p = p
-                        _X.update(p.dictOfFixedFuncs)
+                        _X.dictOfFixedFuncs = p.dictOfFixedFuncs
+                        _X._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
+                        _X._dictOfStochVars = p._dictOfStochVars
+                        _X.maxDistributionSize = p.maxDistributionSize
 
-                r = vstack([[fun(xx) for xx in XX] if funcs[i] in p.unvectorizableFuncs else fun(X).T for i, fun in enumerate(Funcs)]).T
+                r = vstack([[fun(_x) for _x in XX] if funcs[i] in p.unvectorizableFuncs else fun(X).T for i, fun in enumerate(Funcs)]).T
                 
                 
 #                X = [p._vector2point(x[i]) for i in range(nXvectors)]
@@ -287,7 +292,15 @@ class nonLinFuncs:
         
         #if userFunctionType == 'f' and p.isObjFunValueASingleNumber and r.size == 1:
             #r = r.item()
-
+        
+        from FuncDesigner import _Stochastic
+        if isinstance(r, _Stochastic) or (isinstance(r, ndarray) and isinstance(r.flat[0], _Stochastic)):
+            p.err('''
+                error in evaluation of p.%s:
+                objective and constraints cannot be directly applied on stochastic variables,
+                you should use functions like mean, std, var, P.''' % userFunctionType
+                )
+        
         if userFunctionType == 'f' and hasattr(p, 'solver') and p.solver.funcForIterFcnConnection=='f' and hasattr(p, 'f_iter') and not getDerivative:
             if p.nEvals['f']%p.f_iter == 0 or nXvectors > 1:
                 p.iterfcn(x, r)
