@@ -1,11 +1,13 @@
 __docformat__ = "restructuredtext en"
-from time import time, clock
+from time import time, clock, localtime
 from numpy import asfarray, nan, ones, all, atleast_1d, any, isnan, \
-array_equal, asscalar, asarray, ndarray, isscalar, matrix, seterr, isinf #where
+array_equal, asscalar, asarray, ndarray, isscalar, seterr, isinf,inf#, where
+import numpy as np
 from setDefaultIterFuncs import stopcase,  SMALL_DELTA_X,  SMALL_DELTA_F, IS_MAX_ITER_REACHED
 from check import check
-from oologfcn import OpenOptException
+from oologfcn import OpenOptException, errSet
 from openopt import __version__ as version
+from result import OpenOptResult
 
 
 ######################
@@ -24,6 +26,11 @@ try:
 except ImportError:
     hasSetproctitleModule = False
     
+try:
+    from FuncDesigner import FuncDesignerException
+except:
+    FuncDesignerException = OpenOptException
+    
 #from openopt.kernel.ooMisc import __solverPaths__
 ConTolMultiplier = 0.8
 
@@ -46,7 +53,9 @@ ConTolMultiplier = 0.8
 
 def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
     #p = copy.deepcopy(p_, memo=None, _nil=[])
+    errSet.clear()
     p = p_
+    p._localtime_started = localtime()
     if len(args) != 0: p.err('unexpected args for p.solve()')
     if hasattr(p, 'was_involved'): p.err("""You can't run same prob instance for twice. 
     Please reassign prob struct. 
@@ -55,7 +64,7 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
 
     if solver_str_or_instance is None:
         if hasattr(p, 'solver'): solver_str_or_instance = p.solver
-        elif 'solver' in kwargs.keys(): solver_str_or_instance = kwargs['solver']
+        elif 'solver' in kwargs: solver_str_or_instance = kwargs['solver']
 
     if type(solver_str_or_instance) is str and ':' in solver_str_or_instance:
         isConverter = True
@@ -78,6 +87,8 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
             for key, value  in solver_str_or_instance.fieldsForProbInstance.items():
                 setattr(p, key, value)
     p.isConverterInvolved = isConverter
+    if isConverter and p.probType not in ['DFP']: #, 'NLLSP']:
+        p.err('converters are temporarily disabled, use native solvers')
 
     old_err = seterr(all= 'ignore')
     
@@ -104,45 +115,24 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
                 solver_params[key] = value
             else:
                 setattr(p.solver, key, value)
-        elif hasattr(p, key):
+        elif hasattr(p, key) or key in ('lb', 'ub'):
             setattr(p, key, value)
-        else: p.warn('incorrect parameter for prob.solve(): "' + str(key) + '" - will be ignored (this one has been not found in neither prob nor ' + p.solver.__name__ + ' solver parameters)')
+        else: 
+            p.warn('incorrect parameter for prob.solve(): "' + str(key) + '" - will be ignored (this one has been not found in neither prob nor ' + p.solver.__name__ + ' solver parameters)')
     if p.probType == 'EIG' and 'goal' in kwargs:
         p.err('for EIG parameter "goal" should be used only in class instance definition, not in "solve" method')
         
-    p.iterValues = EmptyClass()
-
-    p.iterCPUTime = []
-    p.iterTime = []
-    p.iterValues.x = [] # iter points
-    p.iterValues.f = [] # iter ObjFunc Values
-    p.iterValues.r = [] # iter MaxResidual
-    p.iterValues.rt = [] # iter MaxResidual Type: 'c', 'h', 'lb' etc
-    p.iterValues.ri = [] # iter MaxResidual Index
-    p.solutions = [] # list of solutions, may contain several elements for interalg and mb other solvers
-    if p._baseClassName == 'NonLin':p.iterValues.nNaNs = [] # number of constraints equal to numpy.nan
-
-
-
-    if p.goal in ['max','maximum']: p.invertObjFunc = True
-
-    #TODO: remove it!
-    p.advanced = EmptyClass()
-
-    p.istop = 0
-    p.iter = 0
-    p.graphics.nPointsPlotted = 0
-    p.finalIterFcnFinished = False
-    #for fn in p.nEvals.keys(): p.nEvals[fn] = 0 # NB! f num is used in LP/QP/MILP/etc stop criteria check
-
-    p.msg = ''
-    if not type(p.callback) in (list,  tuple): p.callback = [p.callback]
-    if hasattr(p, 'xlabel'): p.graphics.xlabel = p.xlabel
-    if p.graphics.xlabel == 'nf': p.iterValues.nf = [] # iter ObjFunc evaluation number
+    setSomeDefaults(p)
     
     T = time()
     C = clock()
-    p._Prepare()
+    try:
+        p._Prepare()
+    except OpenOptException as ooe:
+        p.err(ooe.msg)
+    except FuncDesignerException as fde:
+        p.err(fde.msg)
+    
     p.initTime = time() - T
     p.initCPUTime = clock() - C
     if p.initTime > 1 or p.initCPUTime > 1:
@@ -202,7 +192,12 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
     elif p.useScaledResidualOutput:
         p.data4TextOutput[-1] = 'log10(MaxResidual/ConTol)'
 
-    if p.showFeas and p.data4TextOutput[-1] != 'isFeasible': p.data4TextOutput.append('isFeasible')
+    if p.showFeas and p.data4TextOutput[-1] != 'isFeasible': 
+        p.data4TextOutput.append('isFeasible')
+    # TODO: rework
+    if p.solver.__name__ == 'interalg' and p.isObjFunValueASingleNumber:
+        p.data4TextOutput = p.data4TextOutput[:1] + \
+        ['f*_distance_estim', 'f*_bound_estim'] + p.data4TextOutput[1:]
     if p.maxSolutions != 1:
         p._nObtainedSolutions = 0
         p.data4TextOutput.append('nSolutions')
@@ -227,7 +222,8 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
 #    p.ff = nan
     #todo : add scaling, etc
     p.primalConTol = p.contol
-    if not p.solver.__name__.startswith('interalg'): p.contol *= ConTolMultiplier
+    if not p.solver.__name__.startswith('interalg'): 
+        p.contol *= ConTolMultiplier
 
     p.timeStart = time()
     p.cpuTimeStart = clock()
@@ -243,13 +239,58 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
     if p.iprint >= 0:
         p.disp('\n' + '-'*25 + ' OpenOpt %s ' % version + '-'*25)
         pt = p.probType if p.probType != 'NLSP' else 'SNLE'
-        s = 'solver: ' +  p.solver.__name__ +  '   problem: ' + p.name + '    type: %s' % pt
-        if p.showGoal: s += '   goal: ' + p.goal
+        s = 'problem: ' + p.name + '   type: %s' % pt
+        if p.showGoal: 
+            s += '    goal: %s' % p.goal
         p.disp(s)
 
+        s = 'solver: ' +  p.solver.__name__ 
+        
+        if p.solver.__name__ == 'interalg':
+            
+            if p.isObjFunValueASingleNumber:
+                if p.fTol is None and p.probType in ('IP', 'ODE'):
+                    p.pWarn('''
+                    please use p.fTol parameter for interalg with this problem;
+                    p.ftol currently will be used instead
+                    ''')
+                    p.fTol = p.ftol
+                if p.fTol is None:
+                    p.fTol = 1e-7
+                    p.warn('''
+                    interalg requires p.fTol value (required objective function tolerance); 
+                    10^-7 will be used'''
+                    )
+                s += '   fTol: %0.1g ' % p.fTol 
+                if p.probType != 'IP':# TODO: implement for IP
+                    s += '   rTol: %0.1g ' % p.rTol 
+            elif p.probType in ('SNLE', 'NLSP'):
+                s += '   maxSolutions: %s ' % p.maxSolutions 
+        p.disp(s)
+
+        s = 'variables: %d' % p.n
+        #TODO: add num of smooth, bool, int, discrete, fixed, mb string & categorical variables
+        p.disp(s)
+
+        if not p.__isNoMoreThanBoxBounded__():
+            s = 'constaints:  '
+            hasLinearC = p.nb != 0 or p.nbeq != 0
+            hasNonLinearC = p.nc != 0 or p.nh != 0
+            if p.hasLogicalConstraints:
+                s += '{%d logical}' % p.nLogicalConstraints
+                if hasLinearC or hasNonLinearC:
+                    s += '   '
+            if hasLinearC:
+                s += 'linear {%d ineq  %d eq}' % (p.nb, p.nbeq)
+                if hasNonLinearC:
+                    s += '   '
+            if hasNonLinearC:
+                s += 'non-linear {%d ineq  %d eq}' % (p.nc, p.nh-p.nLogicalConstraints)
+            p.disp(s)
+
     p.extras = {}
 
-    
+#    solver(p)
     try:
 #        if 0 and isConverter:
 #            pass
@@ -259,22 +300,7 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
         nErr = check(p)
         if nErr: p.err("prob check results: " +str(nErr) + "ERRORS!")#however, I guess this line will be never reached.
         if p.probType not in ('IP', 'EIG'): p.iterfcn(p.x0)
-        if hasSetproctitleModule:
-            try:
-                originalName = setproctitle.getproctitle()
-                if originalName.startswith('OpenOpt-'):
-                    originalName = None
-                else:
-                    s = 'OpenOpt-' + p.solver.__name__
-                    # if p.name != 'unnamed':
-                    s += '-' + p.name
-                    setproctitle.setproctitle(s)
-            except:
-                originalName = None
-        else:
-            p.pWarn('''
-            please install setproctitle module 
-            (it's available via easy_install and Linux soft channels like apt-get)''')
+        originalName = setProcTitle(p)
         
 #        import os, cProfile
 #        print '----'
@@ -314,16 +340,24 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
 #        p.ff = p.objFuncMultiple2Single(p.fk)
 
         if p.istop == 0: p.istop = 1000
+    except OpenOptException as ooe:
+        p.istop = -1
+        p.err(ooe.msg)
+    except FuncDesignerException as fde:
+        p.err(fde.msg)
     finally:
         if p.isFDmodel:
             for v in p.freeVarsSet | p.fixedVarsSet:
                 if v.fields != ():
                     v.domain, v.aux_domain = v.aux_domain, v.domain
         seterr(**old_err)
-        
+
     if hasSetproctitleModule and originalName is not None:
         setproctitle.setproctitle(originalName)
-    
+    if len(errSet) != 0:
+        finalGUIroutines(p)
+        p._immutable = True
+        return None
     ############################
     p.contol = p.primalConTol
 
@@ -333,10 +367,14 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
             p.iterfcn(p._bestPoint)
         except isSolved:
             pass
+            
     if p.probType != 'EIG':
-        if not hasattr(p, 'xf') and not hasattr(p, 'xk'): p.xf = p.xk = ones(p.n)*nan
-        if hasattr(p, 'xf') and (not hasattr(p, 'xk') or array_equal(p.xk, p.x0)): p.xk = p.xf
-        if not hasattr(p,  'xf') or all(isnan(p.xf)): p.xf = p.xk
+        if not hasattr(p, 'xf') and not hasattr(p, 'xk'): 
+            p.xf = p.xk = ones(p.n)*nan
+        if hasattr(p, 'xf') and (not hasattr(p, 'xk') or array_equal(p.xk, p.x0)): 
+            p.xk = p.xf
+        if not hasattr(p,  'xf') or all(isnan(p.xf)): 
+            p.xf = p.xk
         if p.xf is nan: 
             p.xf = p.xk = ones(p.n)*nan
         
@@ -347,6 +385,7 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
         p.isFeasible = True # check it!
     
     p.isFinished = True # After the feasibility check above!
+    p._localtime_finished = localtime()
     
     if p.probType == 'MOP':
         p.isFeasible = True
@@ -359,18 +398,21 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
         if type(p.ff) == ndarray and p.ff.size == 1:
             p.ff = p.fk = asscalar(p.ff)
         
-    if not hasattr(p,  'ff') or any(p.ff==nan): 
+    if not hasattr(p,  'ff') or any(isnan(p.ff)): 
         p.iterfcn, tmp_iterfcn = lambda *args: None, p.iterfcn
         p.ff = p.fk
         p.iterfcn = tmp_iterfcn
 
-    if p.invertObjFunc:  p.fk, p.ff = -p.fk, -p.ff
+    if p.invertObjFunc:  
+        p.fk, p.ff = -p.fk, -p.ff
 
-    if asfarray(p.ff).size > 1: p.ff = p.objFuncMultiple2Single(p.fk)
+    if asfarray(p.ff).size > 1: 
+        p.ff = p.objFuncMultiple2Single(p.fk)
 
     #p.ff = p.objFuncMultiple2Single(p.ff)
     #if not hasattr(p, 'xf'): p.xf = p.xk
-    if type(p.xf) in (list, tuple) or isscalar(p.xf): p.xf = asarray(p.xf)
+    if type(p.xf) in (list, tuple) or isscalar(p.xf): 
+        p.xf = asarray(p.xf)
     p.xf = p.xf.flatten()
     p.rf = p.getMaxResidual(p.xf) if p.probType not in ('IP', 'ODE') else p.rk
 
@@ -388,6 +430,9 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
     else: 
         p.fk = p.ff
         p.iterfcn(p.xf, p.ff, p.rf)
+        
+    p.lb, p.ub = p._lb, p._ub
+    checkImplicitBounds(p)
 
     p.__finalize__()
     if not p.storeIterPoints: delattr(p.iterValues, 'x')
@@ -404,24 +449,40 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
         p.x0 = p._x0
 
     finalTextOutput(p, r)
-    if not hasattr(p, 'isManagerUsed') or p.isManagerUsed == False: 
-        finalShow(p)
+    finalGUIroutines(p)
+    
+    finish = p.finish if type(p.finish) in (list, tuple, set) else [p.finish]
+    for f in finish:
+        f(r, p)
+    p._immutable = True
+    
     return r
-
-##################################################################
+    
+##########################################################
+def finalGUIroutines(p):
+    if p.isManagerUsed: 
+        p.GUI_items['Quit'].config(state='normal')
+        p.GUI_items['time'].set('%d' % (time() - p.timeStart))
+        p.GUI_items['cputime'].set('%d' % (clock() - p.cpuTimeStart))
+    elif p.plot:
+        finalShow(p)
+        
+##########################################################
 def finalTextOutput(p, r):
     if type(p.msg) == ndarray: # from fmincon etc
         msg = asarray(p.msg.flatten(),int).tolist()
         p.msg = ''.join(chr(c) for c in msg)
+    if p.msg == '':
+        msg = 'finished'
     if p.iprint >= 0:
         if len(p.msg):  
             p.disp("istop: " + str(r.istop) + ' (' + p.msg +')')
         else: 
             p.disp("istop: " + str(r.istop))
 
-        p.disp('Solver:   Time Elapsed = ' + str(r.elapsed['solver_time']) + ' \tCPU Time Elapsed = ' + str(r.elapsed['solver_cputime']))
+        p.disp('Solver:   Time Elapsed = %0.3g  \tCPU Time Elapsed = %0.3g' % (r.elapsed['solver_time'], r.elapsed['solver_cputime']))
         if p.plot:
-            p.disp('Plotting: Time Elapsed = '+ str(r.elapsed['plot_time'])+ ' \tCPU Time Elapsed = ' + str(r.elapsed['plot_cputime']))
+            p.disp('Plotting: Time Elapsed = %0.3g  \tCPU Time Elapsed = %0.3g' % (r.elapsed['plot_time'], r.elapsed['plot_cputime']))
         
         if p.probType == 'MOP':
             msg = '%d solutions have been obtained' % len(p.solutions.coords)
@@ -429,10 +490,9 @@ def finalTextOutput(p, r):
             return
         
         # TODO: add output of NaNs number in constraints (if presernt)
-        if p.useScaledResidualOutput: 
-            rMsg = 'max(residuals/requiredTolerances) = %g' % (r.rf / p.contol)
-        else:
-            rMsg = 'MaxResidual = %g' % r.rf
+        rMsg = 'max(residuals/requiredTolerances) = %g' % \
+        (r.rf / p.contol) if p.useScaledResidualOutput else 'MaxResidual = %g' % r.rf
+
         if not p.isFeasible:
             nNaNs = (len(where(isnan(p.c(p.xf)))[0]) if p.c is not None and type(p.c)!=list else 0) \
             + (len(where(isnan(p.h(p.xf)))[0]) if p.h is not None and type(p.h)!=list else 0)
@@ -445,13 +505,13 @@ def finalTextOutput(p, r):
             p.disp('NO FEASIBLE SOLUTION has been obtained (%s%s, objFunc = %0.8g)' % (nNaNsMsg,  rMsg, r.ff))
         else:
             if p.maxSolutions == 1:
-                msg = "objFunValue: " + (p.finalObjFunTextFormat % r.ff)
+                msg = "objFuncValue: " + (p.finalObjFunTextFormat % r.ff)
                 if not p.isUC: msg += ' (feasible, %s)' % rMsg
             else:
                 msg = '%d solutions have been obtained' % len(p.solutions)
             p.disp(msg)
 
-##################################################################
+##########################################################
 def finalShow(p):
     if not p.plot: return
     pylab = __import__('pylab')
@@ -461,90 +521,77 @@ def finalShow(p):
 #        if os.fork():
             pylab.show()
 
-class OpenOptResult: 
-    # TODO: implement it
-    #extras = EmptyClass() # used for some optional output
-    def __call__(self, *args):
-        if not self.isFDmodel:
-            raise OpenOptException('Is callable for FuncDesigner models only')
-        r = []
-        for arg in args:
-            tmp = [(self._xf[elem] if isinstance(elem,  str) \
-                    else self.xf[elem]) for elem in (arg.tolist() if isinstance(arg, ndarray) \
-                    else arg if type(arg) in (tuple, list) else [arg])]
-            tmp = [asscalar(item) if type(item) in (ndarray, matrix) and item.size == 1 \
-                   #else item[0] if type(item) in (list, tuple) and len(item) == 0 \
-                   else item for item in tmp]
-            r.append(tmp if type(tmp) not in (list, tuple) or len(tmp)!=1 else tmp[0])
-        r = r[0] if len(args) == 1 else r
-        if len(args) == 1 and type(r) in (list, tuple) and len(r) >1: r = asfarray(r, dtype = type(r[0]))
-        return r
-        
-    def __init__(self, p):
-        self.rf = asscalar(asarray(p.rf))
-        self.ff = asscalar(asarray(p.ff))
-        self.isFDmodel = p.isFDmodel
-        self.probType = p.probType
-        if p.probType == 'EIG':
-            self.eigenvalues, self.eigenvectors = p.eigenvalues, p.eigenvectors
-        
-        if p.isFDmodel:
-            from FuncDesigner import oopoint
-            self.xf = dict((v, asscalar(val) if isinstance(val, ndarray) and val.size ==1 \
-                            else dict((field, v.domain[int(val)][j]) for j, field in enumerate(v.fields)) if v.fields != ()\
-                            else v.reverse_aux_domain[int(val)] if 'reverse_aux_domain' in v.__dict__\
-                            else v.aux_domain[val] if 'aux_domain' in v.__dict__ else val) \
-                            for v, val in p.xf.items())
-            if not hasattr(self, '_xf'):
-                #self._xf = dict([(v.name, asscalar(val) if isinstance(val, ndarray) and val.size ==1 else val) for v, val in p.xf.items()])
-                self._xf = dict((v.name, val) for v, val in self.xf.items())
-            self.xf = oopoint(self.xf, maxDistributionSize = p.maxDistributionSize, skipArrayCast = True)
-        else:
-            self.xf = p.xf
-        
-        # TODO: mb use the fields in MOP
-        if p.probType == 'MOP':
-            for attr in ('xf', 'ff', 'rf', '_xf'):
-                delattr(self, attr)
+##########################################################
+def checkImplicitBounds(p):
+    if p.solver._requiresFiniteBoxBounds and p.implicitBounds is not None:
+        delta = 0.01 * (p.implicitBounds[1] - p.implicitBounds[0])
+        tmp = where(isinf(p.lb), p.implicitBounds[0], -inf)
+        delta_r = p.xk - tmp
+        tmp = where(isinf(p.ub), p.implicitBounds[1], inf)
+        delta_l = tmp - p.xk
+        undefined_x = np.all(np.isnan(p.xk))# & np.isnan(p.xf))
+        if not undefined_x and (any(delta_r<delta) or any(delta_l<delta)):
+            Msg = '''
+            some coordinates of solution are too close to implicitBounds
+            (difference is less than 1%)
+            '''
+            p.warn(Msg)
+            if p.istop > 0:
+                p.istop = p.stopcase = 0 
+                p.msg += ';' + Msg
 
-        
-        #if len(p.solutions) == 0 and p.isFeas(p.xk): p.solutions = [p.xk]
-        
-        # TODO: mb perform check on each solution for more safety?
-        # although it can slow down calculations for huge solutions number
-        #self.solutions = p.solutions 
+##########################################################
+def setProcTitle(p):
+    originalName = None
+    if hasSetproctitleModule:
+        try:
+            originalName = setproctitle.getproctitle()
+            if originalName.startswith('OpenOpt-'):
+                originalName = None
+            else:
+                s = 'OpenOpt-' + p.solver.__name__
+                # if p.name != 'unnamed':
+                s += '-' + p.name
+                setproctitle.setproctitle(s)
+        except:
+            pass
+    else:
+        p.pWarn('''
+        please install setproctitle module 
+        (it's available via easy_install 
+        and Linux software channels like apt-get)''')
+    return originalName
 
-        self.elapsed = dict()
-        self.elapsed['solver_time'] = round(100.0*(time() - p.timeStart))/100.0
-        self.elapsed['solver_cputime'] = round(100.0*(clock() - p.cpuTimeStart))/100.0
-        self.elapsed['initialization_time'] = round(100.0*p.initTime)/100.0
-        self.elapsed['initialization_cputime'] = round(100.0*p.initCPUTime)/100.0
+##########################################################
+def setSomeDefaults(p):
+    p.iterValues = EmptyClass()
+    p.iterCPUTime = []
+    p.iterTime = []
+    p.iterValues.x = [] # iter points
+    p.iterValues.f = [] # iter ObjFunc Values
+    p.iterValues.r = [] # iter MaxResidual
+    p.iterValues.rt = [] # iter MaxResidual Type: 'c', 'h', 'lb' etc
+    p.iterValues.ri = [] # iter MaxResidual Index
+    p.solutions = [] # list of solutions, may contain several elements for interalg and mb other solvers
+    if p._baseClassName == 'NonLin':
+        p.iterValues.nNaNs = [] # number of constraints equal to numpy.nan
 
-        for fn in ('ff', 'istop', 'duals', 'isFeasible', 'msg', 'stopcase', 'iterValues',  'special', 'extras', 'solutions'):
-            if hasattr(p, fn):  setattr(self, fn, getattr(p, fn))
+    if p.goal in ['max','maximum']: p.invertObjFunc = True
 
-        if hasattr(p.solver, 'innerState'):
-            self.extras['innerState'] = p.solver.innerState
+    #TODO: remove it!
+    p.advanced = EmptyClass()
 
-        self.solverInfo = dict()
-        for fn in ('homepage',  'alg',  'authors',  'license',  'info', 'name'):
-            self.solverInfo[fn] =  getattr(p.solver,  '__' + fn + '__')
+    p.istop = 0
+    p.iter = 0
+    p.graphics.nPointsPlotted = 0
+    p.finalIterFcnFinished = False
+    #for fn in p.nEvals.keys(): p.nEvals[fn] = 0 # NB! f num is used in LP/QP/MILP/etc stop criteria check
 
-            # note - it doesn't work for len(args)>1 for current Python ver  2.6
-            #self.__getitem__ = c # = self.__call__
-            
-        if p.plot:
-            #for df in p.graphics.drawFuncs: df(p)    #TODO: include time spent here to (/cpu)timeElapsedForPlotting
-            self.elapsed['plot_time'] = round(100*p.timeElapsedForPlotting[-1])/100 # seconds
-            self.elapsed['plot_cputime'] = p.cpuTimeElapsedForPlotting[-1]
-        else:
-            self.elapsed['plot_time'] = 0
-            self.elapsed['plot_cputime'] = 0
-
-        self.elapsed['solver_time'] -= self.elapsed['plot_time']
-        self.elapsed['solver_cputime'] -= self.elapsed['plot_cputime']
-
-        self.evals = dict([(key, val if type(val) == int else round(val *10) /10.0) for key, val in p.nEvals.items()])
-        self.evals['iter'] = p.iter
-        
+    p.msg = ''
+    if not type(p.callback) in (list,  tuple): 
+        p.callback = [p.callback]
+    if hasattr(p, 'xlabel'): 
+        p.graphics.xlabel = p.xlabel
+    if p.graphics.xlabel == 'nf': 
+        p.iterValues.nf = [] # iter ObjFunc evaluation number
 
