@@ -1,23 +1,35 @@
-__docformat__ = "restructuredtext en"
+import os,sys
+sys.path.append(os.getcwd()+os.sep+'kernel')
+
+from LP import LP as CLP
+from LCP import LCP as CLCP
+from EIG import EIG as CEIG
+from SDP import SDP as CSDP
+from QP import QP as CQP
+from MILP import MILP as CMILP
+from STAB import STAB as CSTAB
+from MCP import MCP as CMCP
+from DSP import DSP as CDSP
+from TSP import TSP as CTSP
+from KSP import KSP as CKSP
+from BPP import BPP as CBPP
+from NSP import NSP as CNSP
+from NLP import NLP as CNLP
+from MOP import MOP as CMOP
+from MINLP import MINLP as CMINLP
+from NLSP import NLSP as CNLSP
+from NLLSP import NLLSP as CNLLSP
+from GLP import GLP as CGLP
+from SLE import SLE as CSLE
+from LLSP import LLSP as CLLSP
+from MMP import MMP as CMMP
+from LLAVP import LLAVP as CLLAVP
+from LUNP import LUNP as CLUNP
+from SOCP import SOCP as CSOCP
+from DFP import DFP as CDFP
+from IP import IP as CIP
+from ODE import ODE as CODE
 
-from kernel.LP import LP as CLP
-from kernel.LCP import LCP as CLCP
-from kernel.SDP import SDP as CSDP
-from kernel.QP import QP as CQP
-from kernel.MILP import MILP as CMILP
-from kernel.NSP import NSP as CNSP
-from kernel.NLP import NLP as CNLP
-from kernel.MINLP import MINLP as CMINLP
-from kernel.NLSP import NLSP as CNLSP
-from kernel.NLLSP import NLLSP as CNLLSP
-from kernel.GLP import GLP as CGLP
-from kernel.SLE import SLE as CSLE
-from kernel.LLSP import LLSP as CLLSP
-from kernel.MMP import MMP as CMMP
-from kernel.LLAVP import LLAVP as CLLAVP
-from kernel.LUNP import LUNP as CLUNP
-from kernel.SOCP import SOCP as CSOCP
-from kernel.DFP import DFP as CDFP
 
 def MILP(*args, **kwargs):
     """
@@ -100,14 +112,189 @@ def LP(*args, **kwargs):
     """
     return CLP(*args, **kwargs)
 
+def STAB(*args, **kwargs):
+    """
+    STAB: constructor for graph stability number problem assignment
+    
+    valid calls are:
+    p = STAB(graph, <params as kwargs>)
+    p = STAB(graph = graph, <params as kwargs>)
+    See also: /examples/stab_*.py
+
+    :Parameters:
+    graph: networkx graph instance
+
+    :Returns:
+    OpenOpt STAB class instance
+
+    Notes
+    -----
+    Solving of OpenOpt STAB problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (python list of nodes)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * interalg (BSD license)
+       * MILP solvers at http://openopt.org/MILP
+    """
+    return CSTAB(*args, **kwargs)
+
+
+def MCP(*args, **kwargs):
+    """
+    MCP: constructor for maximum clique problem assignment
+    
+    valid calls are:
+    p = MCP(graph, <params as kwargs>)
+    p = MCP(graph = graph, <params as kwargs>)
+    See also: /examples/mcp_*.py
+
+    :Parameters:
+    graph: networkx graph instance
+
+    :Returns:
+    OpenOpt MCP class instance
+
+    Notes
+    -----
+    Solving of OpenOpt MCP problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (python list of nodes)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * interalg (BSD license)
+       * MILP solvers at http://openopt.org/MILP
+    """
+    return CMCP(*args, **kwargs)
+
+def DSP(*args, **kwargs):
+    """
+    DSP: constructor for domminating set problem assignment
+    
+    valid calls are:
+    p = DSP(graph, <params as kwargs>)
+    p = DSP(graph = graph, <params as kwargs>)
+    See also: /examples/mcp_*.py
+
+    :Parameters:
+    graph: networkx graph instance
+
+    :Returns:
+    OpenOpt DSP class instance
+
+    Notes
+    -----
+    Solving of OpenOpt DSP problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (python list of nodes)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * interalg (BSD license)
+       * MILP solvers at http://openopt.org/MILP
+    """
+    return CDSP(*args, **kwargs)
+    
+def TSP(*args, **kwargs):
+    """
+    TSP: constructor for traveling salesman problem assignment
+    
+    valid calls are:
+    p = TSP(graph, <params as kwargs>)
+    p = TSP(graph = graph, <params as kwargs>)
+    See also: /examples/tsp_*.py
+
+    :Parameters:
+    graph: networkx graph instance
+
+    :Returns:
+    OpenOpt TSP class instance
+
+    Notes
+    -----
+    Solving of OpenOpt TSP problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (python list of edges)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * interalg (BSD license)
+       * MILP solvers at http://openopt.org/MILP
+    """
+    return CTSP(*args, **kwargs)
+
+
+def KSP(*args, **kwargs):
+    """
+    KSP: constructor for knapsack problem assignment
+    
+    valid calls are:
+    p = KSP(objective, objects, <params as kwargs>)
+    See also: /examples/ksp_*.py
+
+    :Parameters:
+    objective: objective (e.g. 'weight', 'cost')
+    objects: Python list or tuple of dictionaries
+
+    :Returns:
+    OpenOpt KSP class instance
+
+    Notes
+    -----
+    Solving of OpenOpt KSP problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (names or numbers of objects)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * MILP solvers at http://openopt.org/MILP
+       * interalg (http://openopt.org/interalg)
+    """
+    return CKSP(*args, **kwargs)
+
+def BPP(*args, **kwargs):
+    """
+    BPP: constructor for traveling salesman problem assignment
+    
+    valid calls are:
+    p = BPP(items, <params as kwargs>)
+    See also: /examples/bpp_*.py
+
+    :Parameters:
+    objective: objective (e.g. 'weight', 'cost')
+    items: Python list or tuple of dictionaries
+
+    :Returns:
+    OpenOpt BPP class instance
+
+    Notes
+    -----
+    Solving of OpenOpt BPP problems is performed via
+    r = p.solve(string_name_of_solver)
+    or r = p.manage(string_name_of_solver) (to enable basic GUI)
+    r.solution - desired solution (names or numbers of objects)
+    r.ff - objFun value (NaN if a problem occured)
+    (see also other r fields)
+    Solvers available for now:  
+       * MILP solvers at http://openopt.org/MILP
+    """
+    return CBPP(*args, **kwargs)
+
 def LCP(*args, **kwargs):
     """
     LCP: constructor for Linear Complementarity Problem assignment
     find w, z: w = Mz + q
     
     valid calls are:
-    p = LP(M, q, <params as kwargs>)
-    p = LP(M=M, q=q, <params as kwargs>)
+    p = LCP(M, q, <params as kwargs>)
+    p = LCP(M=M, q=q, <params as kwargs>)
     See also: /examples/lcp_*.py
 
     :Parameters:
@@ -130,6 +317,36 @@ def LCP(*args, **kwargs):
     """
     return CLCP(*args, **kwargs)
 
+def EIG(*args, **kwargs):
+    """
+    EIG: constructor for Eigenvalues Problem assignment
+    to solve standard eigenvalue problem:
+        find eigenvalues and eigenvectors of square matrix A:   
+            A x = lambda x
+    or general eigenvalue problem:
+        A x = lambda M x
+    
+    valid calls are:
+    p = EIG(M, q, <params as kwargs>)
+    p = EIG(M=M, q=q, <params as kwargs>)
+    See also: /examples/eig_*.py
+
+    :Parameters:
+    A, (optional) M: numpy array or scipy sparse matrix of size n x n
+
+    :Returns:
+    OpenOpt EIG class instance
+
+    Notes
+    -----
+    Solving of EIGs is performed via
+    r = p.solve(string_name_of_solver)
+    see http://openopt.org/EIG for more info
+    Solvers available for now:
+    arpack (license: BSD) 
+
+    """
+    return CEIG(*args, **kwargs)
 
 def SDP(*args, **kwargs):
     """
@@ -362,6 +579,7 @@ def NLP(*args, **kwargs):
     unconstrained:
         scipy_bfgs, scipy_cg, scipy_ncg, 
         (these ones cannot handle user-provided gradient) scipy_powell and scipy_fmin 
+        amsg2p - requires knowing fOpt (optimal value)
     box-bounded:
         scipy_lbfgsb, scipy_tnc - require scipy installed
         bobyqa - doesn't use derivatives; requires http://openopt.org/nlopt installed
@@ -369,9 +587,10 @@ def NLP(*args, **kwargs):
     all constraints:
         ralg
         ipopt (requires ipopt + pyipopt installed)
-        scipy_slsqp (requires scipy from svn 25-Dec-2007 or later)
+        scipy_slsqp
         scipy_cobyla (this one cannot handle user-supplied gradients)
         lincher (requires CVXOPT QP solver),
+        gsubg - for large-scaled problems
         algencan (ver. 2.0.3 or more recent, very powerful constrained solver, GPL,
         requires ALGENCAN + Python interface installed,
         see http://www.ime.usp.br/~egbirgin/tango/)
@@ -398,7 +617,9 @@ def NSP(*args, **kwargs):
     Same usage as NLP (see help(NLP) and /examples/nsp_*.py), but default values of contol, xtol, ftol, diffInt may differ
     Also, default finite-differences derivatives approximation stencil is 3 instead of 1 for NLP
     Solvers available for now:
-        ralg - all constraints, medium-scale (nVars = 1...1000), can handle user-provided gradient/subgradient
+        ralg - all constraints, medium-scaled (nVars = 1...1000), can handle user-provided gradient/subgradient
+        amsg2p - requires knowing fOpt (optimal value), medium-scaled (nVars = 1...1000), can handle user-provided gradient/subgradient
+        gsubg - for large-scaled problems
         scipy_fmin - a Nelder-Mead simplex algorithm implementation, cannot handle constraints and derivatives
         sbplx  -  A variant of Nelder-Mead algorithm; requires http://openopt.org/nlopt installed
         ShorEllipsoid (unconstrained for now) - small-scale, nVars=1...10, requires r0: ||x0-x*||<=r0
@@ -406,6 +627,30 @@ def NSP(*args, **kwargs):
     return CNSP(*args, **kwargs)
 
 def NLSP(*args, **kwargs):
+    """
+    Solving systems of n non-linear equations with n variables
+    Parameters and usage: same as NLP
+    (see help(NLP) and /examples/nlsp_*.py)
+    Solvers available for now:
+        scipy_fsolve (can handle df);
+        converter to NLP. Example: r = p.solve('nlp:ipopt');
+        nssolve (primarily for non-smooth and noisy funcs; can handle all types of constraints and 1st derivatives df,dc,dh; splitting equations to Python list or tuple is recommended to speedup calculations)
+    (these ones below are very unstable and can't use user-supplied gradient - at least, for scipy 0.6.0)
+        scipy_anderson
+        scipy_anderson2
+        scipy_broyden1
+        scipy_broyden2
+        scipy_broyden3
+        scipy_broyden_generalized
+    """
+    r = CNLSP(*args, **kwargs)
+    r.pWarn('''
+    OpenOpt NLSP class had been renamed to SNLE 
+    (system of nonlinear equations), use "SNLE" instead of "NLSP"
+    ''')
+    return r
+    
+def SNLE(*args, **kwargs):
     """
     Solving systems of n non-linear equations with n variables
     Parameters and usage: same as NLP
@@ -437,7 +682,28 @@ def NLLSP(*args, **kwargs):
         converter to NLP. Example: r = p.solve('nlp:ralg')
     """
     return CNLLSP(*args, **kwargs)
+    
+def MOP(*args, **kwargs):
+    '''
+    Multiobjective optimization
+    Search for weak or strong Pareto front
+    
+    Solvers available for now:
+        interalg (http://openopt.org/interalg)
+    '''
+    return CMOP(*args, **kwargs)
 
+def IP(*args, **kwargs):
+    """
+    Integrate a function f: R^n -> R over a given domain lb_i <= x_i <= ub_i
+    """
+    return CIP(*args, **kwargs)
+
+def ODE(*args, **kwargs):
+    """
+    Solve ODE dy/dt = f(y,t), y(0) = y0
+    """
+    return CODE(*args, **kwargs)
 
 def SLE(*args, **kwargs):
     """
@@ -501,6 +767,7 @@ def GLP(*args, **kwargs):
         de (this is temporary name, will be changed till next OO release v. 0.22), license: BSD, requires finite lb <= x <= ub, can handle Ax<=b, c(x) <= 0. The solver is based on differential evolution and made by Stepan Hlushak.
         stogo and mlsl - can use derivatives; require http://openopt.org/nlopt installed
         isres - can handle any constraints; requires http://openopt.org/nlopt installed
+        interalg - exact optimum wrt required tolerance, see http://openopt.org/interalg for details
     """
     return CGLP(*args, **kwargs)
 
