@@ -65,7 +65,8 @@ class cplex(baseSolver):
                 rows,  cols,  vals = Find(q)
                 quad_expr = CPLEX.SparseTriple(ind1=rows, ind2=cols, val = vals)
                 lin_expr = CPLEX.SparsePair(ind=np.arange(np.atleast_1d(u).size), val=u)
-                P.quadratic_constraints.add(quad_expr = quad_expr, lin_expr = lin_expr, rhs = -v if isscalar(v) else -asscalar(v))
+                P.quadratic_constraints.add(quad_expr = quad_expr, lin_expr = lin_expr, 
+                                            rhs = -v if np.isscalar(v) else -np.asscalar(v))
 
         X = np.nan*np.ones(p.n)
         if p.intVars in ([], (), None):
