@@ -310,7 +310,9 @@ def processConstraints(C, C0, y, e, _s, _in, p, dataType, activeCons):
 #        print('len(new_active_cons):',len(new_active_cons))
         if len(inactiveConsIdx) != 0:
             if p.iprint >= 0:
-                s = 'removing %d inactive constraints' % len(inactiveConsIdx)
+                l = len(inactiveConsIdx)
+                s = 'removing %d inactive constraint%s (from %d)' % \
+                (l,'' if l==1 else 's', len(C))
                 if p.iter == 1:
                     s += ' in interalg initialization phase'
                 print(s)
