@@ -426,7 +426,11 @@ def func1(
             # (however, it's costly)
             # !!!!!!!!
             
-            delta = 1.0/n #PythonMax(n, 2)
+            if 1:
+                delta = 1.0/n #PythonMax(n, 2)
+            else:
+#                delta = 
+                pass
             
 #            case = 'new'
 #            if case == 'prev' and p.probType == 'MOP':
@@ -446,14 +450,21 @@ def func1(
                 TNHL_curr_min = where(logical_or(tnlh_curr_1 < tnlh_curr_2, isnan(tnlh_curr_2)), tnlh_curr_1, tnlh_curr_2)
                 t = nanargmin(TNHL_curr_min, 1)
             
-            
+            # 1. Processing objective
             T = nlh_obj_fixed
             s_obj = nanmin(vstack((T[w, t], T[w, n+t])), 0)
             
+#            if p.probType != 'ODE':
+#                # TODO: mb use min/max instead of nanmin/nanmax
+#                maxa, mino = nanmax(a, axis=1), nanmin(o, axis=1) #TODO: mb improve it (computate it wrt modL/modU)
+#                ind_insufficient_obj_diff = maxa - mino < p.fTol
+            
+            # 2. Processing constraints
             #TODO: SKIP nlhc IF ALL ELEMENr38 ARE _constraintInactiveValue (OR mb if activeCons are empty lists)
             T = nlhc
             s_c = nanmin(vstack((T[w, t], T[w, n+t])), 0)
             
+            # 3. Gathering results
             s_obj_prev, s_c_prev = np.real(_s_prev), np.imag(_s_prev)
             ind_problematic_c = logical_and(s_c_prev  <= s_c + delta, s_c != p.solver._constraintInactiveValue)
             
@@ -478,18 +489,19 @@ def func1(
         bs = e[r10_problematic] - y[r10_problematic]
         t[r10_problematic] = np.argmax(bs, 1) 
         
-        # TODO: handle MOP here (& mb IP and ODE)
+        # TODO: mb handle IP and ODE here
         skipProbs = ('IP', 'ODE')
+        # TODO: mb handle case p.solver.dataHandling == 'sorted' here as well
         if 1 and p.probType not in skipProbs and p.solver.dataHandling == 'raw':
-            aor20 = a[r10_problematic] - o[r10_problematic]
+            ind_ignore_obj = logical_not(ind_problematic_obj[r10_problematic])
+            
+            if p.probType != 'MOP':
+                aor20 = a[r10_problematic] - o[r10_problematic]
 #            diff_mod_l, diff_mod_u = aor20[:, :n], aor20[:, n:]
 #            max_ao_diff = where(diff_mod_l > diff_mod_u, diff_mod_l, diff_mod_u) # TODO: mb check for NaNs here
 #            Max_ao_diff = np.max(max_ao_diff, axis=1)
-            Max_ao_diff = np.nanmax(aor20, axis=1)#TODO: use max or nanmax?
-            
-            
-            ind_ignore_obj = logical_not(ind_problematic_obj[r10_problematic])
-            if p.probType != 'MOP':
+                Max_ao_diff = np.nanmax(aor20, axis=1)#TODO: use max or nanmax?
+                
                 r36mall_delta = Max_ao_diff < p.fTol # TODO: mb handle rTol here as well
                 ind_ignore_obj |= r36mall_delta
             #TODO: mb it can be reduced via skipping r36mall_delta processing
@@ -497,16 +509,18 @@ def func1(
             #can be vectorized, but difference is insufficient
             constraints_dep_dict = p._constraints_dep_dict
             freeVarsDict = p._freeVarsDict
+            objdep = p._objDep
             
             for j, i in enumerate(r10_problematic):
                 if ind_ignore_obj[j]:
                     dep = set()
-                elif len(p._objDep) != n:
-                    dep = p._objDep.copy()
+                elif len(objdep) != n:
+                    dep = objdep.copy()
                 else:
                     continue
                 C = [constraints_dep_dict[c_id] for c_id in activeCons[i]]
                 dep.update(*C)
+#                if len(dep) == 0: continue # TODO: is it required? mb rework it?
                 inds_active = array([freeVarsDict[v] for v in dep])
                 _ind = np.argmax(bs[j, inds_active])
                 _i = inds_active[_ind]
@@ -833,14 +847,15 @@ class si:
         self._constraints_reduction = p.solver._constraints_reduction
         for i in range(len(fields)):
             setattr(self, fields[i], args[i])
+    
     def __del__(self):
-        tmp = getattr(self, 'activeCons', [])
-        if len(tmp) == 0: 
-            return
-        
         
         if self._constraints_reduction not in (1,  True):
             return
+            
+        tmp = getattr(self, 'activeCons', [])
+        if len(tmp) == 0: 
+            return
         
         constraints_counter = self._constraints_counter
 #        print('1. constraints_counter:',constraints_counter)
