@@ -1,6 +1,6 @@
 PythonSum = sum
 from numpy import isnan, array, atleast_1d, asarray, logical_and, all, logical_or, any, \
-arange, vstack, inf, logical_not, take, abs, hstack, empty, \
+arange, vstack, inf, logical_not, take, abs, empty, \
 isfinite, argsort, ones, zeros, log1p, array_split#where
 
 # for PyPy
@@ -15,10 +15,11 @@ from interalgLLR import *
 def r43_seq(Arg):
     targets_vals, targets_tols, solutionsF, lf, uf = Arg
     lf, uf = asarray(lf), asarray(uf)
-    if lf.size == 0 or len(solutionsF) == 0: return None
+    if lf.size == 0 or len(solutionsF) == 0: 
+        return None
 
-    m = len(lf)
-    n = lf.shape[2]/2
+    m = lf.shape[0]
+    n = lf.shape[2]//2
     r = zeros((m, 2*n))
 
     for _s in solutionsF:
@@ -76,15 +77,7 @@ def r43_seq(Arg):
                 Tmp = 1.0 - (_uf - _lf) / r20
                 
                 tmp[ind] *= Tmp
-                #raise('unimplemented yet')    
-#            if any(tmp<0) or any(tmp>1):
-#                raise 0
-        
-        # debug
-#        s1, s2 = tmp[tmp==0].size, tmp[tmp==1].size
-#        s1, s2 = s1 / float(tmp.size), s2 / float(tmp.size)
-#        print 'size 0: %0.2f   size 1: %0.2f  rest: %0.2f' % (s1, s2,  1-s1-s2)
-        # debug end
+
         new = 0
         if new:
             ind_0 = tmp == 0.0
@@ -124,15 +117,19 @@ def r43(targets, SolutionsF, lf, uf, pool, nProc):
         r = [elem for elem in result if elem is not None]
         return vstack(r)
 
-
 def r14MOP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, nNodes,  \
          r41, fTol, Solutions, varTols, _in, dataType, \
-         maxNodes, _s, indTC, xRecord):
+         maxNodes, _s, indTC, xRecord, activeCons):
 
     assert p.probType == 'MOP'
     
-    if len(p._discreteVarsNumList):
-        y, e, _s, indTC = adjustDiscreteVarBounds(y, e, _s, indTC, p)
+    asdf1 = [t.func for t in p.targets]
+    
+#    if len(p._discreteVarsNumList):
+#        y, e, ind_trunc = adjustDiscreteVarBounds(y, e, p)
+#        if ind_trunc.size:
+#            
+#            _s, indTC, 
     
     
     if p.nProc != 1 and getattr(p, 'pool', None) is None:
@@ -144,42 +141,61 @@ def r14MOP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, nNodes,
     targets = p.targets # TODO: check it
     m, n = y.shape
     ol, al = [[] for k in range(m)], [[] for k in range(m)]
+    ind_nan = np.zeros(m, bool)
     for i, t in enumerate(targets):
-        o, a, definiteRange, exactRange = func82(y, e, vv, t.func, dataType, p)
+        y, e, o, a, definiteRange, exactRange, _s_, indTC_ = func82(y, e, vv, t.func, dataType, p)
+#        o, a, definiteRange, exactRange, _s, indTC = func82(y, e, vv, t.func, dataType, p, _s, indTC)
         o, a = o.reshape(2*n, m).T, a.reshape(2*n, m).T
+        ind_nan |= logical_and(all(isnan(o), axis=1), np.all(isnan(a), axis=1))
         for j in range(m):
             ol[j].append(o[j])
             al[j].append(a[j])
         #ol.append(o.reshape(2*n, m).T.tolist())
         #al.append(a.reshape(2*n, m).T.tolist())
 
-    nlhf = r43(targets, Solutions.F, ol, al, p.pool, p.nProc)
-    
     fo_prev = 0
-    # TODO: remove NaN nodes here
-
+    
     if y.size == 0:
         return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
     
-    nodes = func11(y, e, nlhc, indTC, residual, ol, al, _s, p)
+    nodes, y, e, o, a, _s, indT, nlhc, residual, activeCons = func11(y, e, nlhc, indTC, residual, ol, al, _s, p, activeCons, ind_nan)
+    if len(nodes) == 0: # after remove some nodes with NaNs
+        return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
+        
+    m, n = y.shape # updated value
+    assert _s.size == m == len(ol) == y.shape[0]
+    
+    nlh_obj = r43(targets, Solutions.F, ol, al, p.pool, p.nProc)
     
     #y, e = func4(y, e, o, a, fo)
     
     
     assert p.solver.dataHandling == 'raw', '"sorted" mode is unimplemented for MOP yet'
     
-    if nlhf is None:
+    if nlh_obj is None:
         new_nodes_tnlh_all = nlhc
     elif nlhc is None: 
-        new_nodes_tnlh_all = nlhf
+        new_nodes_tnlh_all = nlh_obj
     else:
-        new_nodes_tnlh_all = nlhf + nlhc
+        new_nodes_tnlh_all = nlh_obj + nlhc
 
-    asdf1 = [t.func for t in p.targets]
     r5F, r5Coords = getr4Values(vv, y, e, new_nodes_tnlh_all, asdf1, C, p.contol, dataType, p) 
+
+    fo = 0 # unused for MOP
     
     nIncome, nOutcome = r44(Solutions, r5Coords, r5F, targets, p.solver.sigma)
-    fo = 0 # unused for MOP
+    
+    p._frontLength = len(Solutions.F)
+    p._nIncome = nIncome
+    p._nOutcome = nOutcome
+    p.iterfcn(p.x0)
+    #print('iter: %d (%d) frontLenght: %d' %(p.iter, itn, len(Solutions.coords)))
+    
+    if p.istop != 0: 
+        return an, g, fo, None, Solutions, xRecord, r41, r40
+        
+    
+
     
     # TODO: better of nlhc for unconstrained probs
 
@@ -192,56 +208,96 @@ def r14MOP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, nNodes,
 
     hasNewParetoNodes = False if nIncome == 0 else True
     if hasNewParetoNodes:
-        ol2 = [node.o for node in an]
-        al2 = [node.a for node in an]
-        nlhc2 = [node.nlhc for node in an]
-        nlhf2 = r43(targets, Solutions.F, ol2, al2, p.pool, p.nProc)
-        tnlh_all = asarray(nlhc2) if nlhf2 is None else nlhf2 if nlhc2[0] is None else asarray(nlhc2) + nlhf2
-    else:
-        tnlh_all = vstack([new_nodes_tnlh_all] + [node.tnlh_all for node in _in]) if len(_in) != 0 else new_nodes_tnlh_all
+        nodesForRecalculation = nodes + _in
+        ol2 = [node.o for node in nodesForRecalculation]
+        al2 = [node.a for node in nodesForRecalculation]
+        nlhc2 = [node.nlhc for node in nodesForRecalculation]
+        nlh_obj2 = r43(targets, Solutions.F, ol2, al2, p.pool, p.nProc)
+        tnlh_all = asarray(nlhc2) if nlh_obj2 is None else nlh_obj2 if nlhc2[0] is None else asarray(nlhc2) + nlh_obj2
         
-    for i, node in enumerate(nodes):
-        node.tnlh_all = tnlh_all[i]
-
-    r10 = logical_not(any(isfinite(tnlh_all), 1))
+        r10 = logical_not(any(isfinite(tnlh_all), 1))
+        if any(r10):
+            ind = where(logical_not(r10))[0]
+            an = [an[i] for i in ind]
+            tnlh_all = take(tnlh_all, ind, axis=0, out=tnlh_all[:ind.size])        
+        work_elems = (an, tnlh_all)
+    else:
+        work_elems = (nodes, new_nodes_tnlh_all)
+    
+    Nodes, Tnlh_all = work_elems
+    r10 = logical_not(any(isfinite(Tnlh_all), 1))
     if any(r10):
         ind = where(logical_not(r10))[0]
-#        an = asarray(an[ind])
-        an = [an[i] for i in ind]
-        tnlh_all = take(tnlh_all, ind, axis=0, out=tnlh_all[:ind.size])
+        for i in where(r10)[0][::-1]:
+            Nodes.pop(i)
+        if hasNewParetoNodes:
+            Tnlh_all = tnlh_all = take(tnlh_all, ind, axis=0, out=tnlh_all[:ind.size])
+            an = nodes + _in 
+        else:
+            Tnlh_all = new_nodes_tnlh_all = take(new_nodes_tnlh_all, ind, axis=0, out=new_nodes_tnlh_all[:ind.size])
     
-#    else:
-#        tnlh_all = hstack([node.tnlh_all for node in an])
+    if len(an) == 0:
+        return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
         
-    T1, T2 = tnlh_all[:, :tnlh_all.shape[1]/2], tnlh_all[:, tnlh_all.shape[1]/2:]
+    
+
+    n = Tnlh_all.shape[1] // 2
+    T1, T2 = Tnlh_all[:, :n], Tnlh_all[:, n:]
     T = where(logical_or(T1 < T2, isnan(T2)), T1, T2)
-    t = nanargmin(T, 1)
+    t = asarray(nanargmin(T, 1), int)
     
     w = arange(t.size)
-    NN = T[w, t].flatten()
+    NN = T[w, t].flatten()# TODO: IS flatten() required here?
 
-    for i, node in enumerate(an):
-        node.tnlh_all = tnlh_all[i]
-        node.tnlh_curr_best = NN[i]
+    for i, node in enumerate(Nodes):
+        # low priority, mb rework/remove it
+        node.tnlh_curr_best = NN[i]# used only for estimation of number of nodes to involve
+        
+    if p.solver.mop_mode == 1:
+        for i, node in enumerate(Nodes):
+            node.tnlh_all = Tnlh_all[i] # TODO: rework/remove it
+    elif len(nodes) != 0:
+        yc, ec = vstack([n.y for n in nodes]), vstack([n.e for n in nodes])
+        oc, ac = vstack([n.o for n in nodes]), vstack([n.a for n in nodes])
+        
+        # !!!!!!! TODO: implement o, a truncution for MOP there !!!!!!
+        indT = func4(p, yc, ec, oc, ac, fo, new_nodes_tnlh_all)
+        
+        _indT = array([n.indtc for n in nodes])
+        indT &= _indT
+        for i, n in enumerate(nodes):
+            n.indtc = indT[i]
+        
         
-    astnlh = argsort(NN)
-    
-#    an = an[astnlh]
-    an = [an[i] for i in astnlh]
 
-    p._t = t
+    astnlh = argsort(NN)
+
+    if hasNewParetoNodes:
+        an = [an[i] for i in astnlh]
+        p._t = t
+    else:# Nodes is nodes
+        nodes = [nodes[i] for i in astnlh]
+        an = nodes + _in
+        tmp = getattr(p, '_t', [])
+       
+        p._t = np.hstack((t, tmp)) if len(tmp) else t
+
+    
+    
+    #assert _s.size == m == len(ol) == y.shape[0]
+    
     
     # TODO: form _s in other level (for active nodes only), to reduce calculations
-    if len(an) != 0:
-        T = asarray([node.nlhf for node in an])
-#        nlhc_fixed = asarray([node.nlhc for node in an])
-        if an[0].nlhc is not None:
-            T += asarray([node.nlhc for node in an])
-#        T = nlhf_fixed + nlhc_fixed if nlhc_fixed[0] is not None else nlhf_fixed 
-        p.__s = \
-        nanmin(vstack(([T[w, t], T[w, n+t]])), 0)
-    else:
-        p.__s = array([])
+#    if len(an) != 0:
+#        T = asarray([node.nlh_obj_fixed for node in an])
+##        nlhc_fixed = asarray([node.nlhc for node in an])
+#        if an[0].nlhc is not None:
+#            T += asarray([node.nlhc for node in an])
+##        T = nlhf_fixed + nlhc_fixed if nlhc_fixed[0] is not None else nlhf_fixed 
+#        p.__s = \
+#        nanmin(vstack(([T[w, t], T[w, n+t]])), 0)
+#    else:
+#        p.__s = array([])
 
 #        p._nObtainedSolutions = len(solutions)
 #        if p._nObtainedSolutions > maxSolutions:
@@ -251,22 +307,16 @@ def r14MOP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, nNodes,
 #            return an, g, fo, None, solutions, coords, xRecord, r41, r40
     
     
-    # TODO: fix it
-    p._frontLength = len(Solutions.F)
-    p._nIncome = nIncome
-    p._nOutcome = nOutcome
-    p.iterfcn(p.x0)
-    #print('iter: %d (%d) frontLenght: %d' %(p.iter, itn, len(Solutions.coords)))
-    
-    if p.istop != 0: 
-        return an, g, fo, None, Solutions, xRecord, r41, r40
-        
+
     #an, g = func9(an, fo, g, p)
 
     nn = maxNodes#1 if asdf1.isUncycled and all(isfinite(o)) and p._isOnlyBoxBounded and not p.probType.startswith('MI') else maxNodes
     
     an, g = func5(an, nn, g, p)
     nNodes.append(len(an))
+    
+    assert _s.size == m == len(ol) == y.shape[0]
+    
     return an, g, fo, _s, Solutions, xRecord, r41, r40
 
 
@@ -282,7 +332,7 @@ def r44(Solutions, r5Coords, r5F, targets, sigma):
     #n = len(r5Coords[0])
     # TODO: mb use inplace r5Coords / r5F modification instead?
     for j in range(m):
-        if isnan(r5F[0][0]):
+        if np.any(isnan(r5F[j])):
             continue
         if Solutions.coords.size == 0:
             Solutions.coords = array(r5Coords[j]).reshape(1, -1)
@@ -356,91 +406,3 @@ def r44(Solutions, r5Coords, r5F, targets, sigma):
                 Solutions.coords = vstack((Solutions.coords, r5Coords[j]))
                 Solutions.F.append(r5F[j])
     return nIncome, nOutcome
-
-
-
-#r43 = lambda targets, SolutionsF, lf, uf: r43_seq([t.val for t in targets], [t.tol for t in targets], SolutionsF, lf, uf)
-    #parallel_r43(targets, SolutionsF, lf, uf, 2)
-    
-
-
-
-
-        
-        
-#def r43(targets, Solutions, lf, uf):
-#    lf, uf = asarray(lf), asarray(uf)
-#    solutionsF = Solutions.F
-#    S = len(solutionsF)
-#    if S == 0: return None
-#    
-#    #print '!', asarray(lf).shape, asarray(uf).shape
-#    #lf, uf = asarray(lf), asarray(uf)
-#
-#    m = len(lf)
-#    n = lf[0][0].size/2
-#    r = zeros((m, 2*n))
-#    
-#    ss = asarray(solutionsF)
-#    tmp = ones((S, m, 2*n))
-#    for i, t in enumerate(targets):
-#        val, tol = t.val, t.tol
-#        
-#        #TODO: mb optimize it
-#        o, a = tile(lf[:, i], (S, 1, 1)), tile(uf[:, i], (S, 1, 1)) 
-#        
-#        if val == inf:
-#            ff = s[i] + tol
-#            ind = a > ff
-#            if any(ind):
-#                t1 = a[ind]
-#                t2 = o[ind]
-#                
-#                # TODO: check discrete cases
-#                Tmp = (ff-t2) / (t1-t2)
-#                Tmp[t1==t2] = 0.0 # for discrete cases
-#                tmp[ind] *= Tmp
-#                tmp[ff<o] = 0.0
-#                
-#        elif val == -inf:
-#            ff = (ss[:, i] - tol).reshape(S, 1, 1)
-#            try:
-#                ind = o < ff
-#            except:
-#                pass
-#            if any(ind):
-#                try:
-#                    t1 = a[ind]
-#                except:
-#                    pass
-#                t2 = o[ind]
-#                # TODO: check discrete cases
-#                Tmp = (t1-ff) / (t1-t2)
-#                Tmp[t1==t2] = 0.0 # for discrete cases
-#                try:
-#                    tmp[ind] *= Tmp
-#                except:
-#                    pass
-#                tmp[a<ff] = 0.0
-#        else: # finite val
-#            ff = abs(s[i]-val) - tol
-#            if ff <= 0:
-#                continue
-#            _lf, _uf = o - val, a - val
-#            ind = logical_or(_lf < ff, _uf > - ff)
-#            _lf = _lf[ind]
-#            _uf = _uf[ind]
-#            _lf[_lf>ff] = ff
-#            _lf[_lf<-ff] = -ff
-#            _uf[_uf<-ff] = -ff
-#            _uf[_uf>ff] = ff
-#            
-#            r20 = a[ind] - o[ind]
-#            Tmp = 1.0 - (_uf - _lf) / r20
-#            Tmp[r20==0] = 0.0 # for discrete cases
-#            tmp[ind] *= Tmp
-#                #raise('unimplemented yet')    
-##            if any(tmp<0) or any(tmp>1):
-##                raise 0
-#        r -= log1p(-sum(tmp, axis=0)) * 1.4426950408889634 # log2(e)
-#    return r
