@@ -359,22 +359,31 @@ def r14(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C,
     p.iterfcn(xRecord, r40)
 
     # TODO: fix it (rTol)
-    if 0 and not isSNLE and isfinite(r41) and len(an):
-        tmp = array([node.key for node in an])
-        f_r16estim = tmp.min()
-        abs_tmp = abs(tmp)
-        abs_f_r16estim = abs(f_r16estim)
+    if 1 and not isSNLE and isfinite(r41) and len(an):
+        nodes_infinum = array([node.key for node in an])
+#        f_r16estim = nodes_infinum.min()
+        abs_nodes_infinum = abs(nodes_infinum)
+#        abs_f_r16estim = abs(f_r16estim)
 #        abs_r40 = abs(r40)
+        abs_r41 = abs(r41)
+        
+        rTolThreshold = p.rTol * where(abs_nodes_infinum < abs_r41, abs_nodes_infinum, abs_r41)
+#        print('rTolThreshold[:2]:', rTolThreshold[:2], 'abs_r41:', abs_r41, 'abs_tmp[:2]:', abs_tmp[:2])
+#        print('f_r16estim:', f_r16estim, 'tmp[:2] - f_r16estim:', tmp[:2] - f_r16estim)
         
         #prev
-        cond_exclude = tmp - f_r16estim <= \
-        p.rTol * where(abs_tmp < abs_f_r16estim, abs_tmp, abs_f_r16estim)
+        cond_exclude = r41 - nodes_infinum < rTolThreshold
+        l = len(where(cond_exclude)[0])
+        if p.debug and l != 0:
+            print('num excluded by rTol: %d from %d' % (l, len(an)))
+        
         g = PythonMin([g] + [an[j].key for j in where(cond_exclude)[0]])
         cond_remain = logical_not(cond_exclude)
         ind = where(cond_remain)[0]
+        g = PythonMin([g] + [an[j].key for j in where(cond_exclude)[0]])
         an = [an[i] for i in ind]
     
-        g = PythonMin([g] + [an[j].key for j in where(cond_exclude)[0]])
+        
         
     if p.istop != 0: 
         return an, g, fo, None, Solutions, xRecord, r41, r40
