@@ -1,17 +1,19 @@
 PythonMin, PythonMax = min, max
+PythonAll = all
 import numpy as np
 
+from sortedcontainers import SortedDict
+
 from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F, MAX_NON_SUCCESS, IS_NAN_IN_X
 from openopt.kernel.baseSolver import *
-from openopt.kernel.MOP import MOPsolutions
 
 from openopt.solvers.UkrOpt.interalgMisc import *
 from FuncDesigner import sum as fd_sum, abs as fd_abs, oopoint
 from ii_engine import *
-from interalgCons import processConstraints
-from interalgODE import interalg_ODE_routine
 
-from interalgLLR import adjustr4WithDiscreteVariables
+
+#from interalgT import getTruncatedArrays
+
 
 bottleneck_is_present = False
 try:
@@ -35,8 +37,9 @@ class interalg(baseSolver):
     dataType = np.float64
     #maxMem = '150MB'
     maxNodes = 150000
+    minActiveNodes = 15
     maxActiveNodes = 150
-    sigma = 0.1 # for MOP, unestablished
+    sigma = 0.1 # for MOP, unestablished yet
     
     _requiresBestPointDetection = True
     
@@ -44,17 +47,35 @@ class interalg(baseSolver):
     p.__isNoMoreThanBoxBounded__() or p.probType in ('MOP', 'IP') #and p.probType != 'IP'
     _requiresFiniteBoxBounds = True
 
+    _constraintInactiveValue = -1000.0 # ~= log2(1e-300), 
+    # a value for _inner_ interalg purposes, not intended for solver users
+    
+    _constraints_reduction = True
+
+    mop_mode = 2
+
     def __init__(self): 
         self.dataHandling = 'auto'
         
     def __solver__(self, p):
-        
+        if not p.isFDmodel:
+            p.err('solver %s can handle only FuncDesigner problems' % self.__name__)
+            
         isMOP = p.probType == 'MOP'
         if isMOP:
             from interalgMOP import r14MOP
         #isOpt = p.probType in ['NLP', 'NSP', 'GLP', 'MINLP']
         isODE = p.probType == 'ODE'
+        isIP = p.probType == 'IP'
         isSNLE = p.probType in ('NLSP', 'SNLE')
+        
+        if not isIP and not isODE:
+            if 1:
+                from interalgCons import processConstraints
+                use_saved = 0
+            else:
+                from Saved_interalgCons import processConstraints
+                use_saved = 1
 
         if not p.__isFiniteBoxBounded__() and not isODE: 
             p.err('''
@@ -68,12 +89,11 @@ class interalg(baseSolver):
 #            p.err('solver %s cannot handle FuncDesigner problems with some variables declared as fixed' % self.__name__)
 #        if p.probType in ('LP', 'MILP'):
 #            p.err("the solver can't handle problems of type " + p.probType)
+
         for v in p.freeVarsSet:
             if v.domain is int or v.domain is 'int':
                 p.err('interalg cannot handle intVars yet, use domain = [val1, val2, ...,val_k]')
-        if not p.isFDmodel:
-            p.err('solver %s can handle only FuncDesigner problems' % self.__name__)
-       
+
         dataType = self.dataType
         if type(dataType) == str:
             if not hasattr(np, dataType):
@@ -83,12 +103,29 @@ class interalg(baseSolver):
             self.dataType = dataType
         
         vv = list(p._freeVarsList)
-        p._freeVarsDict = dict((v, i) for i, v in enumerate(vv))
-        x0 = dict((v, p._x0[v]) for v in vv)
-        domain = oopoint([(v, [atleast_1d(p.lb[i]), atleast_1d(p.ub[i])]) for i,  v in enumerate(vv)], skipArrayCast = True)
-        domain.dictOfFixedFuncs = p.dictOfFixedFuncs
         
-        isIP = p.probType == 'IP'
+        #TODO: create it in openopt kernel, not in interalg kernel
+        p._freeVarsDict = dict((v, i) for i, v in enumerate(vv))
+        
+        
+        x0 = dict((v, p._x0[v]) for v in vv)
+        domain = oopoint(((v, [atleast_1d(p.lb[i]), atleast_1d(p.ub[i])]) for i,  v in enumerate(vv)), skipArrayCast = True)
+        domain.dictOfFixedFuncs = p.dictOfFixedFuncs
+        domain._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
+        domain._dictOfStochVars = p._dictOfStochVars
+        domain._p = p
+        domain.maxDistributionSize = p.maxDistributionSize
+        
+#                domain = ooPoint(domainData, skipArrayCast=True)
+#        domain.isMultiPoint = True
+#        domain.nPoints = m
+#        domain.dictOfFixedFuncs = p.dictOfFixedFuncs
+#        domain._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
+#        domain.maxDistributionSize = p.maxDistributionSize
+#        domain._dictOfStochVars = p._dictOfStochVars
+#        domain._p = p
+        
+        
         if isIP:
             pb = r14IP
             p._F = asarray(0, self.dataType)
@@ -101,6 +138,11 @@ class interalg(baseSolver):
             pb = r14MOP
         else:
             pb = r14
+            if not isSNLE:
+                f_dep = p.user.f[0].dep
+                dep = f_dep & p.freeVarsSet
+                p._continuous_obj_dep_variables = PythonAll(v.domain is None for v in dep)
+                
 
         
         for val in p._x0.values():
@@ -124,12 +166,12 @@ class interalg(baseSolver):
         p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
         p.kernelIterFuncs.pop(MAX_NON_SUCCESS, None)
         
-        if not bottleneck_is_present and not isODE:
-                p.pWarn('''
-                installation of Python module "bottleneck" 
-                (http://berkeleyanalytics.com/bottleneck,
-                available via easy_install, takes several minutes for compilation)
-                could speedup the solver %s''' % self.__name__)
+#        if not bottleneck_is_present and not isODE:
+#                p.pWarn('''
+#                installation of Python module "bottleneck" 
+#                (http://berkeleyanalytics.com/bottleneck,
+#                available via easy_install, takes several minutes for compilation)
+#                could speedup the solver %s''' % self.__name__)
         
         n = p.n
         
@@ -168,10 +210,17 @@ class interalg(baseSolver):
 
         if fTol is None and not isMOP: # TODO: require tols for MOP
             fTol = 1e-7
-            p.warn('solver %s require p.fTol value (required objective function tolerance); 10^-7 will be used' % self.__name__)
+            p.warn('''
+            solver %s requires p.fTol value (required objective function tolerance); 
+            10^-7 will be used''' % self.__name__
+            )
 
         xRecord = 0.5 * (lb + ub)
-        adjustr4WithDiscreteVariables(xRecord.reshape(1, -1), p)
+        
+        # TODO: rework it if discrete variables will be somehow added for IP or ODE
+        if not isIP and not isODE:
+            from interalgLLR import adjustr4WithDiscreteVariables
+            adjustr4WithDiscreteVariables(xRecord.reshape(1, -1), p)
 
         r40 = np.inf
         
@@ -191,11 +240,15 @@ class interalg(baseSolver):
             
             # temporary change, TODO: rework it
             asdf1.resolveSchedule = asdf1._getDep()
+            p._objDep = asdf1.Dep & p.freeVarsSet
             
             # TODO: check it, for reducing calculations
             #C.update([elem == 0 for elem in p.user.f])
         elif isMOP:
             asdf1 = p.user.f
+            p._objDep = set()
+            p._objDep.update(*[_f.Dep for _f in asdf1])
+            p._objDep &= p.freeVarsSet
             Solutions.F = []
             if point(p.x0).isFeas(altLinInEq=False):
                 Solutions.solutions.append(p.x0.copy())
@@ -204,7 +257,7 @@ class interalg(baseSolver):
                 p._solutions = Solutions
         elif not isODE:
             asdf1 = p.user.f[0]
-            
+            p._objDep = asdf1.Dep & p.freeVarsSet
             #if p.fOpt is not None:  fOpt = p.fOpt
             if p.goal in ('max', 'maximum'):
                 asdf1 = -asdf1
@@ -227,6 +280,9 @@ class interalg(baseSolver):
                 if p.fOpt > r41:
                     p.warn('user-provided fOpt seems to be incorrect, ')
                 r41 = p.fOpt
+        else:# ODE
+            # TODO: implement p._objDep = asdf1.Dep & p.freeVarsSet here
+            assert 0, 'unimplemented yet'
 
 #        if isSNLE:
 #            if self.dataHandling == 'raw':
@@ -259,7 +315,7 @@ class interalg(baseSolver):
 
                     # !!!!!!!!!!!!!!!!!!!! check it - mb 2nd condition is incorrect
                     #if isinstance(c, BooleanOOFun) and not isinstance(c, SmoothFDConstraint): continue
-                    if hasattr(c,'_unnamedBooleanOOFunNumber'):
+                    if c.discrete:#hasattr(c,'_unnamedBooleanOOFunNumber'):
                         continue
                     
                     r = func.interval(domain, self.dataType)
@@ -276,19 +332,57 @@ class interalg(baseSolver):
                     p.disp('interalg parameter dataHandling has been autoselected to "%s"' % self.dataHandling)
 
             #self.dataHandling = 'sorted' if isIP or (p.__isNoMoreThanBoxBounded__() and n < 50) else 'raw'
-
+        
+        if self.dataHandling == 'sorted':
+            self._constraints_reduction = False
+        
         self.maxActiveNodes = int(self.maxActiveNodes)
 #        if self.maxActiveNodes < 2:
 #            p.warn('maxActiveNodes should be at least 2 while you have provided %d. Setting it to 2.' % self.maxActiveNodes)
         self.maxNodes = int(self.maxNodes)
         
         g = np.inf
-        C = p._FD.nonBoxConsWithTolShift
-        C0 = p._FD.nonBoxCons
+        
+#        A = getattr(p, 'A', None)
+#        if A is not None:
+#            p.A2 = [line for line in p.A]
+#            p.b2 = p.b.tolist()
+        
+        # TODO: improve C & C0, omit unused data, check for their processing in openopt kernel
+        _C = p._FD.nonBoxConsWithTolShift
+        _C0 = p._FD.nonBoxCons
         
         if isSNLE:
-            C += [(elem==0, elem, -(elem.tol if elem.tol != 0 else p.ftol), (elem.tol if elem.tol != 0 else p.ftol)) for elem in p.user.f]
-            C0 += [(elem==0, elem, 0, 0, (elem.tol if elem.tol != 0 else p.ftol)) for elem in p.user.f]
+# !!!!!!!!!!!!!!!! TODO: connect linear constraints to Aeq & beq matrices
+            _C += [(elem==0, elem, -(elem.tol if elem.tol != 0 else p.ftol), (elem.tol if elem.tol != 0 else p.ftol)) for elem in p.user.f]
+            _C0 += [(e[0], e[1], 0, 0, (e[1].tol if e[1].tol != 0 else p.ftol)) for e in _C]
+            
+
+        if 1:
+            C = SortedDict((elem[0]._id, elem) for elem in _C)
+            C0 = SortedDict((elem[0]._id, elem) for elem in _C0)        
+        else:
+            C = dict((elem[0]._id, elem) for elem in _C)
+            C0 = dict((elem[0]._id, elem) for elem in _C0)        
+        
+        p._constraints_dep_dict = dict((k, elem[1].Dep & p.freeVarsSet) for k, elem in C.items())
+        
+#        if not use_saved:
+
+        p._constraints_counter = dict((c_id, 0) for c_id in C0.keys())
+        activeCons = [list(C0.keys())]
+#        else:
+##            C, C0 = _C, _C0
+#            activeCons = [[elem[0]._id for elem in _C0]]
+#            p._constraints_counter = dict()
+        
+#        C.sort(key=lambda elem: elem[0]._id)
+#        C0.sort(key=lambda elem: elem[0]._id) # we could use sort order from prev sort but computation time difference is insufficient
+        
+#        c_id = dict((elem[0]._id, j) for j, elem in enumerate(C0))
+        
+        # TODO: mb use list or ndarray instead
+        
         
         # TODO: hanlde fixed variables here
         varTols = p.variableTolerances
@@ -312,50 +406,83 @@ class interalg(baseSolver):
         _s = atleast_1d(inf)
         
         if isODE or (isIP and p.n == 1):
+            from interalgODE import interalg_ODE_routine
             interalg_ODE_routine(p, self)
             return
         
         #_in = np.array([], object)
         _in = []
+        
+#        print('p.iter:', p.iter)
         while 1:
+            
+            assert len(activeCons) == _s.size
+#            p.iprint=1
+            
             if len(C0) != 0: # SNLE also can have constraints
-                y, e, nlhc, residual, definiteRange, indT, _s = processConstraints(C0, y, e, _s, p, dataType)
+                if use_saved:
+                    y, e, nlhc, residual, definiteRange, indT, \
+                    _s =\
+                    processConstraints(_C0, y, e, _s, p, dataType)
+                else:
+                    y, e, nlhc, residual, definiteRange, indT, \
+                    _s, activeCons =\
+                    processConstraints(C, C0, y, e, _s, _in, p, dataType, activeCons)
             else:
-                nlhc, residual, definiteRange, indT = None, None, True, None
-#            P = array([-0.63521194458007812, -0.3106536865234375, 0.0905609130859375, 0.001522064208984375, -0.69999999999999996, -0.99993896484375, 0.90000152587890625, 1.0, 4.0])
-#            print('-', p.iter, hasPoint(y, e, P), pointInd(y, e, P))
+                residual, definiteRange, indT = None, True, None
+                nlhc = np.empty((y.shape[0], 2*y.shape[1]), np.float64)
+                nlhc.fill(self._constraintInactiveValue)
+#            a = np.array([-1.26291003e-02,  3.01864599e-02,   1.89144034e-03,  -2.78295240e-04, -2.16195926e-05,   7.54278724e-04,  -4.66201550e-04,  -4.93097371e-05, 3.18665265e-04,  -7.38596373e-04])
+#            print('-', p.iter, hasPoint(y, e, a), pointInd(y, e, a))
+
+            assert len(activeCons) == _s.size == y.shape[0]
+            
             if y.size != 0:
                 an, g, fo, _s, Solutions, xRecord, r41, r40 = \
                 pb(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, \
                              nNodes, r41, fTol, Solutions, varTols, _in, \
-                             dataType, maxNodes, _s, indT, xRecord)
+                             dataType, maxNodes, _s, indT, xRecord, activeCons)
                 if _s is None:
-                    break
+                    break 
+                
             else:
                 an = _in
+                # TODO: mb remove fo recalculation from here
                 fo = 0.0 if isSNLE or isMOP else \
                 PythonMin(r41, \
                           r40 - (fTol if Solutions.maxNum == 1 else 0.0))
+            # TODO: check it                            ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+            
             pnc = PythonMax(pnc, 
                                     len(np.atleast_1d(an) if type(an) == np.ndarray else an))
             
-            y, e, _in, _s = \
+#            assert len(activeCons) == _s.size == y.shape[0]
+            
+            
+            y, e, _in, _s, activeCons = \
                 func12(an, self.maxActiveNodes, p, Solutions, vv, varTols, np.inf if isIP else fo)
-
-            nActiveNodes.append(y.shape[0]/2)
+            
+            nActiveNodes.append(y.shape[0]//2)
             if y.size == 0: 
                 if len(Solutions.coords) > 1:
                     p.istop, p.msg = 1001, 'all solutions have been obtained'
                 else:
                     p.istop, p.msg = 1000, 'solution has been obtained'
                 break            
-            ############# End of main cycle ###############
+        '''                        ^^^ End of main cycle ^^^                        '''
             
         if not isSNLE and not isIP and not isMOP:
-            if p._bestPoint.betterThan(p.point(p.xk)):
-                p.iterfcn(p._bestPoint)
-            else:
-                p.iterfcn(p.xk)
+            p_xk = p.point(p.xk)
+            p.iterfcn(p._bestPoint if p._bestPoint.betterThan(p_xk) else p_xk)
+            
+            if isscalar(r40):
+                p.extras['r40'] = r40
+            if isscalar(r41):
+                p.extras['r41'] = r41
+            if isscalar(g):
+                p.extras['g'] = g
+            
+
         
         ff = p.fk # ff may be not assigned yet
 #        ff = p._bestPoint.f()
@@ -370,12 +497,12 @@ class interalg(baseSolver):
         if not isFeas and p.istop > 0:
             p.istop, p.msg = -1000, 'no feasible solution has been obtained'
         
-        if isSNLE:
+        if isSNLE or isMOP:
             lf = inf
         else:
             lf = array([t.key for t in an])#.flatten()
             if lf.size != 0:
-                g = nanmin([nanmin(lf), g])
+                g = nanmin((nanmin(lf), g))
 
         if not isMOP:
 
@@ -400,7 +527,12 @@ class interalg(baseSolver):
             p.extras['extremumBounds'] = tmp if not isIP else 'unimplemented for IP yet'
         
         if isMOP or p.maxSolutions != 1:
-            p.solutions = (MOPsolutions if isMOP else list)([p._vector2point(s) for s in Solutions.coords])
+            tmp = [p._vector2point(s) for s in Solutions.coords]
+            if isMOP:
+                # TODO: mb rework it
+                from openopt.kernel.MOP import MOPsolutions 
+                tmp = MOPsolutions(tmp)
+            p.solutions = tmp
         if isMOP:
             for i, s in enumerate(p.solutions):
                 s.useAsMutable = True
@@ -421,7 +553,7 @@ class interalg(baseSolver):
         if p.iprint >= 0 and not isMOP:
             s = 'Solution with required tolerance %0.1e \n is%s guarantied' \
             %(fTol, '' if p.extras['isRequiredPrecisionReached'] else ' NOT') \
-            if p.probType not in ('NLP', 'GLP', 'NSP')\
+            if p.probType not in ('NLP', 'GLP', 'NSP', 'MINLP')\
             else 'Solution with required tolerances fTol = %g, rTol = %g \n is%s guarantied' \
             %(fTol, p.rTol, '' if p.extras['isRequiredPrecisionReached'] else ' NOT')
             
@@ -434,5 +566,3 @@ class Solution:
     pass
     
 
-    
-
